__author__ = "Unai Saralegui"
__copyright__ = "Copyright 2019, Unai Saralegui"
__credits__ = ["Unai Saralegui", "Josu Gomez"]
__license__ = "MIT"
__version__ = "0.1"
__maintainer__ = "Unai Saralegui"
__email__ = "usaralegui@gmail.com"
__status__ = "Development"


# FIELDS KEYWORDS
ACTION = "action"
CONTENT = "content"
FORMAT = "format"
INSTRUMENT = "instrument"
RECORD = "record"
REPORT_ID = "report_id"
TOKEN = "token"
TYPE = "type"
DATE_RANGE_BEGIN = "dateRangeBegin"
DATE_RANGE_END = "dateRangeEnd"
FILTER_LOGIC = "filterLogic"

# ACTION keywords
ACTION_EXPORT = "export"

# CONTENT keywords
CONTENT_ARM = "arm"
CONTENT_EVENT = "event"
CONTENT_FIELD_NAMES = "exportFieldNames"
CONTENT_FILE = "file"
CONTENT_FORM_EVENT_MAPPING = "formEventMapping"
CONTENT_INSTRUMENT = "instrument"
CONTENT_METADATA = "metadata"
CONTENT_PARTICIPANT_LIST = "participantList"
CONTENT_PDF = "pdf"
CONTENT_PROJECT = "project"
CONTENT_PROJECT_XML = "project_xml"
CONTENT_RECORD = "record"
CONTENT_REPORT = "report"
CONTENT_USER = "user"
CONTENT_VERSION = "version"

# FORMAT keywords
FORMAT_JSON = "json"
FORMAT_XML = "xml"
